import logging

from django.apps import AppConfig
from django.db.models.signals import post_migrate

logger = logging.getLogger(__name__)

class DrfIamConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "drf_iam"

    def ready(self):
        if not hasattr(self, 'already_loaded'):
            from drf_iam.utils.load_viewset_permissions import load_permissions_from_urls
            post_migrate.connect(
                load_permissions_from_urls,
                dispatch_uid="drf_iam.utils.load_permissions_from_urls",
            )
            self.already_loaded = True


import inspect
from django.urls.resolvers import URLPattern, URLResolver
from rest_framework.viewsets import ViewSetMixin
from rest_framework.decorators import action

from django.urls import get_resolver
from ..models import Policy


def is_viewset(cls):
    return isinstance(cls, type) and issubclass(cls, ViewSetMixin)


def get_viewset_actions(viewset_cls):
    actions = set()

    # Default ViewSet methods
    default_actions = {'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy'}
    for action_name in default_actions:
        if hasattr(viewset_cls, action_name):
            actions.add(action_name)

    # Custom @action methods
    for name, method in inspect.getmembers(viewset_cls, predicate=inspect.isfunction):
        if hasattr(method, 'mapping'):
            for http_method in method.mapping:
                actions.add(f"{name}")

    return actions


def extract_viewsets_from_urlpatterns(urlpatterns, prefix=''):
    for pattern in urlpatterns:
        if isinstance(pattern, URLResolver):
            # Recurse into included URLConfs
            yield from extract_viewsets_from_urlpatterns(pattern.url_patterns, prefix + str(pattern.pattern))
        elif isinstance(pattern, URLPattern):
            callback = pattern.callback
            cls = getattr(callback, 'cls', None)
            if is_viewset(cls):
                yield {
                    'prefix': prefix,
                    'pattern': pattern.pattern,
                    'viewset': cls,
                    'callback': callback
                }


def load_permissions_from_urls(**kwargs):
    urlpatterns = get_resolver().url_patterns
    for entry in extract_viewsets_from_urlpatterns(urlpatterns):
        viewset_cls = entry['viewset']
        basename = getattr(viewset_cls,"iam_policy_name") or viewset_cls.__name__.lower().replace('viewset', '')
        actions = get_viewset_actions(viewset_cls)

        for action in actions:
            full_action = f"{basename}:{action}"
            Policy.objects.get_or_create(
                action=full_action,
                resource_type=basename,
                defaults={
                    'description': f"Permission for {full_action} on {basename}"
                }
            )



# Generated by Django 5.2 on 2025-04-19 11:30

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Role",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
                ("description", models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name="Policy",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("action", models.CharField(max_length=100)),
                ("resource_type", models.CharField(max_length=100)),
                ("description", models.TextField(blank=True)),
            ],
            options={
                "verbose_name": "policy",
                "verbose_name_plural": "policies",
                "ordering": ["action", "resource_type"],
                "unique_together": {("action", "resource_type")},
            },
        ),
        migrations.CreateModel(
            name="RolePolicy",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "policy",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="drf_iam.policy"
                    ),
                ),
                (
                    "role",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="drf_iam.role"
                    ),
                ),
            ],
            options={
                "verbose_name": "role policy",
                "verbose_name_plural": "role policies",
                "ordering": ["role", "policy"],
                "unique_together": {("role", "policy")},
            },
        ),
        migrations.AddField(
            model_name="role",
            name="policies",
            field=models.ManyToManyField(
                blank=True,
                db_index=True,
                related_name="policies",
                related_query_name="policies",
                through="drf_iam.RolePolicy",
                to="drf_iam.policy",
            ),
        ),
    ]



from setuptools import setup, find_packages

setup(
    name='drf-iam',
    version='0.1.0',
    packages=find_packages(),  # Automatically finds all sub-packages
    include_package_data=True,  # Include non-Python files from MANIFEST.in
    install_requires=[
        'Django>=3.2',
        'djangorestframework>=3.12'
    ],
    author='Tushar Patel',
    author_email='tushar.patel@gmail.com',
    description='IAM-style roles and permissions for Django Rest Framework',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',  # Important if using Markdown
    url='https://github.com/tushar1328/drf-iam.git',  # Optional but helpful
    project_urls={
        'Documentation': 'https://drf-iam.readthedocs.io/en/latest/installation.html',
        'Source': 'https://github.com/tushar1328/drf-iam.git',
    },
    classifiers=[
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',  # Or your license
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',  # Or your minimum supported Python version
)
